function est = run_filter_Multi(model,meas,theta_in)


%---

%=== Setup

theta_in(1)=deg2rad(theta_in(1));
theta_in(3)=deg2rad(theta_in(3));
%output variables
est.X= cell(meas.K,1);
est.N= zeros(meas.K,1);
est.L= cell(meas.K,1);
est.f_z= cell(meas.K,1);%ǹʽ7.28
est.f_all=0;%ǹʽ7.26
%filter parameters
filter.J_max= 100000;                                         %total number of particles
filter.J_target=1000;%0427 1000;                                        % 1000  generated number of particles per expected target
filter.J_birth= model.L_birth*filter.J_target;                %generated number of particles from birth intensity

filter.run_flag= 'silence';            %'disp' or 'silence' for on the fly output

est.filter= filter;

%=== Filtering 

%initial prior
w_update= [1; 1; 1; 1];
m_init= [400 0 -800 -200;0 0 0 0;-600 0 200 800;0 0 0 0];
P_init= diag([100 10 100 10]).^2;
x_update= m_init;

%recursive filtering
for k=1:meas.K
    %---prediction 
    pS_vals= compute_pS(model,x_update); pS_vals= pS_vals(:);
    qS_vals= 1-pS_vals;

    x_predict = gen_newstate_fn(model,x_update,'noise');                                                                %surviving particles
    w_predict = pS_vals.*w_update;                                                                                      %surviving weights

    x_predict= cat(2,gen_gms(model.w_birth,model.m_birth,model.P_birth,filter.J_birth),x_predict);                      %append birth particles
    w_predict= cat(1,sum(model.w_birth)*ones(filter.J_birth,1)/filter.J_birth,w_predict);                               %append birth weights                                                
        
    %---update
    %number of measurements
    m= size(meas.Z{k},2);
    pD_vals= compute_pD_Multi(k,model,x_predict); pD_vals= pD_vals(:);
    qD_vals= 1-pD_vals;

    %missed detection weight
    pseudo_likelihood= qD_vals;
    temp_z=1;
    if m~=0
        %m detection weights
        for ell=1:m
            meas_likelihood= compute_likelihood_Multi(k,model,meas.Z{k}(:,ell),x_predict,theta_in)';
            pseudo_likelihood = pseudo_likelihood+pD_vals.*meas_likelihood/(model.lambda_c*model.pdf_c + (pD_vals.*meas_likelihood)'*w_predict);
            temp_z=temp_z*(model.lambda_c*model.pdf_c + sum((pD_vals.*meas_likelihood)'*w_predict));
        end
    end
    w_update= pseudo_likelihood.*w_predict;
    x_update= x_predict;    
            
    %---for diagnostics
    w_posterior= w_update;
    
    %---resampling
    J_rsp= min(ceil(sum(w_update)*filter.J_target),filter.J_max);
    idx= randsample(length(w_update),J_rsp,true,w_update); %idx= resample(w_update,J_rsp);
    w_update= sum(w_update)*ones(J_rsp,1)/J_rsp;
    x_update= x_update(:,idx);
 %% f(Z_k|Z_1:k-1,theta)
     est.f_z{k}=   exp(-sum(pD_vals.*w_predict))*temp_z;%ĳһkʱ̵7.28
     est.f_all=est.f_all+log(est.f_z{k});
 
 %% 
    
    
    %--- state extraction   
    if sum(w_update) > 0.5
        [x_c,I_c]= our_kmeans(x_update,w_update,1);
        est.N(k)= 0;
        for j=1:size(x_c,2);
            if sum(w_update(I_c{j})) > .5,
                est.X{k}= [ est.X{k} x_c(:,j) ];
                est.N(k)= est.N(k)+1;
                est.L{k}= [];
            end
        end
    else
        est.N(k)= 0; est.X{k}= [];
    end
  
    %---display diagnostics
    if ~strcmp(filter.run_flag,'silence')
        disp([' time= ',num2str(k),...
         ' #est mean=' num2str(sum(w_update),4),...
         ' #est card=' num2str(est.N(k),4),...
         ' Neff_updt= ',num2str(round(1/sum((w_posterior/sum(w_posterior)).^2)))...
         ' Neff_rsmp= ',num2str(round(1/sum((w_update/sum(w_update)).^2)))   ]);
    end

end

            